// Humidity controlled bathroom fan - Firmware
// This code was written by and it's property of Andrea Giudici.
// See www.area51staff.it for more info


#include <dht11.h>
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#define DHT11PIN 13

dht11 DHT11;
LiquidCrystal_I2C lcd(0x20,20,4);

unsigned long time;
float h,t;
boolean fan=false;

short timeout=60; // Time in seconds between high humidity and fan switch (led blinking time)
unsigned long timer_start=0;
boolean timer_active=false;


void setup()
{
  lcd.init();
  time=millis();
  
  pinMode(10,OUTPUT);
  pinMode(9,OUTPUT);
  digitalWrite(9,LOW);
  h=0;
  t=0;
 
}

void loop()
{
  if(millis()>time+1000){
    time=millis();
    doStuff();
  }
  
  
}

void doStuff() {
  int chk = DHT11.read(DHT11PIN);
  
  lcd.setCursor(0,0);
  switch (chk)
  {
    case DHTLIB_OK: 
                doDisplay();
                doFan();
		break;
    case DHTLIB_ERROR_CHECKSUM: 
		lcd.print("Checksum error"); 
		break;
    case DHTLIB_ERROR_TIMEOUT: 
		lcd.print("Time out error"); 
		break;
    default: 
		lcd.print("Unknown error"); 
		break;
  }
  
  lcd.setCursor(0,1);
  
  lcd.print("Fan ");
  
  if(timer_start!=0 && !fan) {
    lcd.print("[");
    lcd.print(((unsigned int)timeout*1000-(millis()-timer_start))/1000);
    lcd.print("]    ");
    digitalWrite(9,HIGH);
    delay(100);
    digitalWrite(9,LOW);
  }
  
  
  
  if(timer_start!=0 && fan)
    lcd.print("[ON]     ");
  
  if(timer_start==0)
    lcd.print("[OFF]     ");
    
lcd.setCursor(0,2);
  lcd.print("Uptime:");
  lcd.print(millis());
  lcd.setCursor(0,3);
  lcd.print("Fourth line!");    
  
}

void doFan() {
  if(h>50.0) {
    if(!timer_active) {
      timer_start=millis();
      timer_active=true;
    }
    else
      if(millis()>timer_start+(unsigned int)timeout*1000) {
        digitalWrite(10,HIGH);
        fan=true;
      }
  }
  else {
    timer_active=false;
    timer_start=0;
    digitalWrite(10,LOW);
    fan=false;
  }
}

void doDisplay() {
    h=(float)DHT11.humidity;
    t=(float)DHT11.temperature;
    lcd.print("H=");
    lcd.print(h);
    lcd.print("%;T=");
    lcd.print(t); 
}
